<?php
/*
** Email Reminders (emrem)
**
** Copyright (C) 2000-2002, Rob Hudson <crh@cogit8.org>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
*/

if(defined("__EMREM_CLASS__")) return;
define("__EMREM_CLASS__", 1);

include 'config.php';

class Emrem 
{

  // Define variables

  var $id;        // auto_incremented id from database
  var $email;     // where to send the reminder
  var $subject;   // Subject of reminder
  var $body;      // Body of reminder
  var $stamp;     // unix timestamp 

  var $query = '';
  var $success = '';
  var $error = '';

  /*************************************************************
   * Constructor
   *************************************************************/

  function Emrem ($id = 0) 
  {
  }

  /*************************************************************
   * int add (email (array), stamp (int), recur (array))
   *************************************************************/

  function add ($email, $stamp, $recur) 
  {
    global $db,$config;

    // Error checking
    list ($day, $month, $year, $hour) = array(date('j', $stamp), date('n', $stamp), date('Y', $stamp), date('G', $stamp));
    if (!checkdate ($month, $day, $year)) 
    {
      $this->error .= '<li>Not a valid date';
    }

    if ($stamp < mktime()) 
    {
      $this->error .= '<li>The date you selected has passed.  Cannot violate laws of physics.';
    }

    if (!$email['subject'] || $email['subject'] == '') 
    {
      $this->error .= '<li>No subject given.';
    }

    if (!$email['email'] || $email['email'] == '') 
    {
      $this->error .= '<li>No e-mail given.';
    }

    // Handle more than one email
    $email['email'] = trim ($email['email']);
    $email['email'] = ereg_replace ("[[:space:]]+", "", $email['email']);
    $emails = explode (",", $email['email']);

    // A simulated foreach for PHP 3 support
    reset($emails);
    while (list(,$val) = each ($emails)) 
    {
      if (!eregi ("^[a-zA-Z0-9_.-]+@[a-zA-Z0-9_-]+\.[a-zA-Z0-9_.-]+$", $val)) 
      {
        $this->error .= "<li>Invalid e-mail address: $val";
      }
    }

    // Check if recur_every is numeric
    if ($recur['every'] != '' && !ereg("^[0-9]*$", $recur['every'])) 
    {
      $this->error .= '<li>Recur every <b>' . $recur['every'] . '</b> should contain only numeric values.';
    }

    if ($error != '') 
    {
      return 0;
    }

    // Everything looks okay, do the real stuff

    // Set cookie to remember email - only the first one if there are more than one
    $expire = time() + 2592000; // 30 days
    setcookie ("cookie_email", $emails[0], $expire);
    
    // Insert reminders into database
    reset($emails);
    while (list(,$val) = each ($emails)) 
    {
      $query = "
        INSERT INTO 
          " . $config['table'] . "
        VALUES 
         (NULL, 
         '$val', 
         '" . $email['subject'] . "', 
         '" . $email['body'] . "', 
         $day, 
         $month, 
         $year, 
         $hour, 
         '" . $recur['every'] . "', 
         '" . $recur['interval'] . "'
         )";
      $this->query .= "$query";

      $result = $db->query($query);
      if (DB::iserror($result)) {
        $this->error = "Error in add:<br>
                        SQL: $query<br>" . 
                        $result->getMessage();
      }
    }

    if ($this->error == '') 
    {
      // Success: Set class values and return positive
      $this->email = $email['email'];
      $this->subject = stripslashes($email['subject']);
      $this->body = stripslashes($email['body']);
      $this->stamp = $stamp;

      // Set success string
      reset($emails);
      while (list(,$val) = each ($emails)) 
      {
        $email_list .= "$val, ";
      }
      $email_list = ereg_replace(", $", "", $email_list);
      $this->success = "
        The following was successfully entered into the reminder database:
        <p><dd>
        <table width=500 cellspacing=0 cellpadding=3 border=0>
        <tr bgcolor=\"" . BOXCOLOR . "\">
         <td valign=top><b>To:</b></td>
         <td>$email_list</td>
        </tr>
        <tr bgcolor=\"" . BOXCOLOR . "\">
         <td valign=top><b>Date:</b></td>
         <td>" . date("l, F j, Y h:00 A", $stamp) . "</td>
        </tr>
        <tr bgcolor=\"" . BOXCOLOR . "\">
         <td valign=top><b>Subject:</b></td>
         <td>$this->subject</td>
        </tr>
        <tr>
         <td valign=top><b>Body:</b></td>
         <td>" . nl2br($this->body) . "</td>
        </tr>
        </table>
        </dd>
      ";
      return 1;
    } 
    else 
    {
      return 0;
    }
  }

  function send ($stamp) 
  {
    global $db, $config;

    list ($day, $month, $year, $hour) = array(date('j', $stamp), date('n', $stamp), date('Y', $stamp), date('G', $stamp));
    $query = "SELECT * FROM reminders WHERE  date_day=$day AND date_month=$month AND date_year=$year AND date_hour=$hour";
    $result = $db->query($query);
    if(DB::iserror($result)) 
    {
      die("Error in send.<br>SQL: $query<br>" . $result->getMessage());
    }

    $num_rows = $result->numRows();

    for ($i = 0; $i < $num_rows; $i++) 
    {
      $row = $result->fetchRow(DB_FETCHMODE_ASSOC);

      $newdate = array();

      $to   = $row['email_to'];
      $body = wordwrap($row['email_body']);
      $subj = $config['mail_subj_prepend'] . ' ' . $row['email_subject'];

      if ($to != '') 
      {
        if (mail($to, $subj, $body, 'From: ' . $config['mail_from'])) 
        {
          echo "\n(ID:{$row['id']}) E-mail Reminder sent to: <$to>";
        }
      }

      // Update record if recurring event
      if ($row['recur_every'] != 0) 
      {
        $every    = $row['recur_every'];
        $interval = $row['recur_interval'];

        switch ($interval) 
        {
          case 'hour':

            $newdate['hour'] = $row['date_hour'];
            $newdate['hour'] += $every;

            if ($newdate['hour'] > 23) 
            {
              $newdate['hour'] -= 24;
              $every = 1;
            } 
            else 
            {
              $every = 0;
            }
            // Falls thru to day check

          case 'day':

            if (!isset($newdate['day'])) 
            {
              $newdate['day'] = $row['date_day'];
            }
            $newdate['day'] += $every;

            if (!checkdate($row['date_month'], $newdate['day'], $row['date_year'])) 
            {
              // Day passed the end of the month,
              // Subtract days in month from day and let every=1 pass thru to month
              $num_days = date('t',mktime(0,0,0,$row['date_month'],1,$row['date_year']));
              $newdate['day'] -= $num_days;
              $every = 1;
            } 
            else 
            {
              $every = 0;
            }

            // Falls thru to month check

          case 'month':

            if (!isset($newdate['month'])) 
            {
              $newdate['month'] = $row['date_month'];
            }
            $newdate['month'] += $every;

            if ($newdate['month'] > 12) 
            {
              $newdate['month'] -= 12;
              $every = 1;
            } 
            else 
            {
              $every = 0;
            }

            // Falls thru to year check

          case 'year':

            if (!isset($newdate['year'])) 
            {
              $newdate['year'] = $row['date_year'];
            }
            $newdate['year'] += $every;

        }

        $query = "UPDATE reminders SET ";
        foreach ($newdate as $key => $val) 
        {
          $query .= "date_$key='$val', ";
        }
        $query = ereg_replace(", $", " WHERE id=" . $row['id'], $query);
        $update_result = $db->query($query);
        if (DB::iserror($update_result)) 
        {
          echo "DB Error on Update: " . $update_result->getMessage();
        }
        else
        {
          echo "\n(ID:{$row['id']}) Updated reminder for <{$row['email_to']}> to send on: ";
          echo date('r', mktime($newdate['hour'], 0, 0, $newdate['month'], $newdate['day'], $newdate['year']));
        }
      } 
      else 
      {
        $query = "DELETE FROM " . $config['table'] . " WHERE id=" . $row['id'];
        $db->query($query);
        echo "\n(ID:{$row['id']}) Deleted reminder.";
      }
    }
  }
} //end class
?>
