#!/usr/bin/perl
#
# Email Reminders
#
# Copyright (C) 2000, Rob Hudson <rob@euglug.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
use Date::Calc qw(Days_in_Month check_date Today_and_Now);
use DBI;

# buffer to write output only if we send mails
my $buffer = '';

# Change this line to your server email
#
my $from = "Email Reminders <reminders\@mundy.org>";

# Database setup: change any required variables below
#
my $host     = 'localhost';
my $database = 'emrem';
my $username = 'root';
my $password = '';

# Shouldn't need to change anything else below
#
my $dbh = DBI->connect("DBI:mysql:$database", $username, $password) || die "Failed DBI connect(): $!\n";

my ($year,$month,$day,$hour,$min,$sec) = Today_and_Now();
my $query = "SELECT * FROM reminders WHERE date_day=$day AND date_month=$month AND date_year=$year AND date_hour=$hour";
my $sth = $dbh->prepare($query);
my $rc  = $sth->execute();

while (my @data = $sth->fetchrow_array) 
{
  my($id,$email_to,$email_subject,$email_body,$date_day,$date_month,$date_year,$date_hour,$recur_every,$recur_interval) = @data;

  $buffer .= "\n (ID:$id) Sending mail to <$email_to>";

  send_email($email_to, $email_subject, $email_body, $from);

  # Update record if recurring event
  if ($recur_every != 0) 
  {
    my $every = $recur_every;
    my $interval = $recur_interval;

    my $fallthru = 0;

    my %newdate = (
      "hour"  => $date_hour,
      "day"   => $date_day,
      "month" => $date_month,
      "year"  => $date_year
    );

    if ($interval eq 'hour') 
    {
      $newdate{'hour'} += $every;

      if ($newdate{'hour'} > 23) 
      {
        $newdate{'hour'} -= 24;
        $every = 1;
        $fallthru = 1;
      } 
      else 
      {
        $every = 0;
      }
    }

    if ($interval eq 'day' || $fallthru == 1) 
    {
      $newdate{'day'} += $every;

      if (!check_date($date_year, $date_month, $newdate{'day'})) 
      {
        # Day passed the end of the month,
        # Subtract days in month from day and let every=1 pass thru to month
        my $num_days = Days_in_Month($date_year,$date_month);
        $newdate{'day'} -= $num_days;
        $every = 1;
        $fallthru = 1;
      } 
      else 
      {
        $every = 0;
      }
    }

    if ($interval eq 'month' || $fallthru == 1) 
    {
      $newdate{'month'} += $every;

      if ($newdate{'month'} > 12) 
      {
        $newdate{'month'} -= 12;
        $every = 1;
        $fallthru = 1;
      } 
      else 
      {
        $every = 0;
        $fallthru = 0;
      }
    }

    if ($interval eq 'year' || $fallthru == 1) 
    {
      $newdate{'year'} += $every;
    }

    # Now that we have valid values for hour, day, month and year
    # Update the record so this reminder is sent again

    $query = "
      UPDATE 
        reminders 
      SET 
        date_hour=$newdate{'hour'},
        date_day=$newdate{'day'},
        date_month=$newdate{'month'},
        date_year=$newdate{'year'}
      WHERE
        id=$id
    ";

    if ($dbh->do($query)) 
    {
      # I want this to list the month as: 01 when month=1, but it currently doesn't
      $buffer .= "\n (ID:$id) Update reminder for <$email_to> (";
      $buffer .= sprintf('%2d',$newdate{'month'}) . "/";
      $buffer .= sprintf('%2d',$newdate{'day'})   . "/";
      $buffer .= sprintf('%2d',$newdate{'year'});
      $buffer .= " @ " . sprintf('%2d',$newdate{'hour'}) . "00 hours)";
    } 
    else 
    {
      $buffer .= "\n Failed to execute query:\n$query";
    }

  } 
  else 
  {
    $query = "DELETE FROM reminders WHERE id=$id";
    if ($dbh->do($query)) 
    {
      $buffer .= "\n (ID:$id) Removed reminder";
    } 
    else 
    {
      $buffer .= "\n Failed to execute query:\n$query";
    }
  }
}

if ($sth->rows > 0) 
{
  print $buffer;
}

undef $sth;
$dbh->disconnect;

sub send_email($$$$) 
{
  my ($to, $subject, $body, $from) = @_;

  $body =~ s/\\//g;
  $to =~ s/@/\@/g;
  open (MAIL, "|/usr/sbin/sendmail -oi -t") or die "Can't fork for sendmail: $!\n";
  print MAIL <<__STOP;
From: $from
To: $to
Subject: $subject

$body
__STOP
  close(MAIL) or warn "mail didn't close nicely";

}

0;
